# Generate devel rpm
%global with_devel 0
# Build project from bundled dependencies
%global with_bundled 0
# Build with debug info rpm
%global with_debug 1
# Run tests in check section
%global with_check 1
# Generate unit-test rpm
%global with_unit_test 0

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif


%global provider        github
%global provider_tld    com
%global project         junegunn
%global repo            fzf
# https://github.com/junegunn/fzf
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}
%global commit          b46227dcb6f1718d66ad828443d9f03a4bd58c4c
%global shortcommit     %(c=%{commit}; echo ${c:0:7})
%global commitdate      20181006
%global realver         0.17.5

Name:           fzf
Version:        0.17.5
Release:        1%{?dist}
Summary:        A command-line fuzzy finder written in Go
License:        MIT
URL:            https://%{provider_prefix}
Source0:        https://%{provider_prefix}/archive/%{realver}/%{repo}-%{realver}.tar.gz
Source1:        README.Fedora

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

BuildRequires: golang(github.com/gdamore/tcell)
BuildRequires: golang(github.com/gdamore/tcell/encoding)
BuildRequires: golang(github.com/mattn/go-isatty)
BuildRequires: golang(github.com/mattn/go-runewidth)
BuildRequires: golang(github.com/mattn/go-shellwords)
BuildRequires: golang(golang.org/x/crypto/ssh/terminal)


%description
fzf is a general-purpose command-line fuzzy finder.

It's an interactive Unix filter for command-line that can be used with any
list; files, command history, processes, hostnames, bookmarks, git commits,
etc.

%if 0%{?with_devel}
%package -n golang-%{provider}-%{project}-%{repo}-devel
Summary:       %{summary}
BuildArch:     noarch

Requires:      golang(github.com/gdamore/tcell)
Requires:      golang(github.com/gdamore/tcell/encoding)
Requires:      golang(github.com/mattn/go-isatty)
Requires:      golang(github.com/mattn/go-runewidth)
Requires:      golang(github.com/mattn/go-shellwords)
Requires:      golang(golang.org/x/crypto/ssh/terminal)

Provides:      golang(%{import_path}/src) = %{version}-%{release}
Provides:      golang(%{import_path}/src/algo) = %{version}-%{release}
Provides:      golang(%{import_path}/src/tui) = %{version}-%{release}
Provides:      golang(%{import_path}/src/util) = %{version}-%{release}

%description -n golang-%{provider}-%{project}-%{repo}-devel
%{summary}

This package contains library source intended for
building other packages which use import path with
%{import_path} prefix.
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%package -n golang-%{provider}-%{project}-%{repo}-unit-test-devel
Summary:         Unit tests for %{name} package
%if 0%{?with_check}
#Here comes all BuildRequires: PACKAGE the unit tests
#in %%check section need for running
%endif

# test subpackage tests code from devel subpackage
Requires:        golang-%{provider}-%{project}-%{repo}-devel = %{version}-%{release}

%if 0%{?with_check} && ! 0%{?with_bundled}
%endif


%description -n golang-%{provider}-%{project}-%{repo}-unit-test-devel
%{summary}

This package contains unit tests for project
providing packages with %{import_path} prefix.
%endif

%prep
%setup -q -n %{repo}-%{realver}
cp %{SOURCE1} .


%build
mkdir -p src/%{provider}.%{provider_tld}/%{project}/
ln -s $(pwd) src/%{provider}.%{provider_tld}/%{project}/%{repo}
export GOPATH=$(pwd):%{gopath}

%gobuild -o bin/fzf %{import_path}

# Cleanup interpreters
sed -i -e '/^#!\//, 1d' shell/completion.*
sed -i -e '1d;2i#!/bin/bash' bin/fzf-tmux


%install
install -d -p %{buildroot}%{_bindir}
install -Dpm0755 bin/fzf %{buildroot}%{_bindir}/
install -Dpm0755 bin/fzf-tmux %{buildroot}%{_bindir}/
install -d -p %{buildroot}%{_mandir}/man1
install -Dpm0644 man/man1/*.1 %{buildroot}%{_mandir}/man1/

install -d %{buildroot}%{_datadir}/fzf

# Install vim plugin
install -d %{buildroot}%{_datadir}/vim/vimfiles/plugin
install -Dpm0644 plugin/fzf.vim %{buildroot}%{_datadir}/vim/vimfiles/plugin/
install -d %{buildroot}%{_datadir}/nvim/site/plugin
install -Dpm0644 plugin/fzf.vim %{buildroot}%{_datadir}/nvim/site/plugin/

# Install shell completion
install -d %{buildroot}%{_sysconfdir}/bash_completion.d
install -Dpm0644 shell/completion.bash %{buildroot}%{_sysconfdir}/bash_completion.d/fzf
install -d %{buildroot}%{_datadir}/zsh/site-functions
install -Dpm0644 shell/completion.zsh %{buildroot}%{_datadir}/zsh/site-functions/fzf

# Install shell key bindings (not enabled)
install -d %{buildroot}%{_datadir}/fzf/shell
install -Dpm0644 shell/key-bindings.* %{buildroot}%{_datadir}/fzf/shell/

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list
# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

# testing files for this project
%if 0%{?with_unit_test} && 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
# find all *_test.go files and generate unit-test-devel.file-list
for file in $(find . -iname "*_test.go") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> unit-test-devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif

%check
%if 0%{?with_check} && 0%{?with_unit_test} && 0%{?with_devel}
%if ! 0%{?with_bundled}
export GOPATH=%{buildroot}/%{gopath}:%{gopath}
%else
# No dependency directories so far

export GOPATH=%{buildroot}/%{gopath}:%{gopath}
%endif

%if ! 0%{?gotest:1}
%global gotest go test
%endif

%gotest %{import_path}/src
%gotest %{import_path}/src/algo
%gotest %{import_path}/src/tui
%gotest %{import_path}/src/util
%endif

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
%license LICENSE
%doc README.md README-VIM.md CHANGELOG.md README.Fedora
%{_bindir}/fzf
%{_bindir}/fzf-tmux
%{_mandir}/man1/fzf.1*
%{_mandir}/man1/fzf-tmux.1*
%dir %{_datadir}/fzf
%{_datadir}/fzf/shell
%dir %{_datadir}/zsh/site-functions
%{_datadir}/zsh/site-functions/fzf
%dir %{_datadir}/vim/vimfiles/plugin
%{_datadir}/vim/vimfiles/plugin/fzf.vim
%dir %{_datadir}/nvim/site/plugin
%{_datadir}/nvim/site/plugin/fzf.vim
%{_sysconfdir}/bash_completion.d/fzf

%if 0%{?with_devel}
%files -n golang-%{provider}-%{project}-%{repo}-devel -f devel.file-list
%license LICENSE
%doc README.md CHANGELOG.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%files -n golang-%{provider}-%{project}-%{repo}-unit-test-devel -f unit-test-devel.file-list
%license LICENSE
%doc README.md CHANGELOG.md
%endif

%changelog
* Wed Oct 10 2018 Elliott Sales de Andrade <quantum.analyst@gmail.com> - 0.17.5-1
- New upstream release.

* Thu Jun 14 2018 Elliott Sales de Andrade <quantum.analyst@gmail.com> - 0.17.4-1
- New upstream release.

* Fri Mar 16 2018 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.17.3-1
- New upstream release.

* Wed Feb 07 2018 Fedora Release Engineering <releng@fedoraproject.org> - 0.17.1-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Sun Oct 22 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.17.1-1
- New upstream release.

* Wed Sep 20 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.17.0.2-1
- New upstream release.

* Mon Sep 4 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.17.0-1
- New upstream release.
- Mention neovim in readme as well.

* Mon Sep 4 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-7
- Install plugin for neovim also.

* Thu Aug 24 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-6
- Turn off source and unit testing packages.

* Wed Aug 23 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-5
- Enable vim plugin by default.
- Fix directory ownership on plugins.

* Mon Aug 21 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-4
- Restore dist tag.

* Sun Aug 20 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-3
- Fix source URL.
- Enable debuginfo subpackage.
- Correct interpreters in shebangs.

* Sun Aug 20 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-2
- Add fzf binary to package.
- Add Fedora-specific readme about optional things.
- Install shell completions.
- Install (disabled) vim plugin also.
- Install (disabled) shell key bindings.

* Fri Aug 18 2017 Elliott Sales de Andrade <quantum.analyst@gmail.com> 0.16.11-1
- Initial package for Fedora
